/************************************************************************/
/* snowalbedo.c                                                         */
/*                                                                      */
/* Albedoof snow, executable.                                           */
/*                                                                      */
/* Author: Arve Kylling, NILU, Norway.                                  */ 
/*                                                                      */
/*----------------------------------------------------------------------*/
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*----------------------------------------------------------------------*/
/************************************************************************/

  
/*  @26c@                

@code{snowalbedo} calculate the diffuse and direct albedo of snow as 
formulated by Wiscombe and Warren, Journal of the Atmospheric Sciences, 
vol, 37, 2712-2733, 1980.

The different options to @code{snowalbedo} are displayed when executing:

@example
snowalbedo -h
@end example

    @c26@ */




#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <ascii.h>
#include <numeric.h>
#include <getopt.h>
#include <table.h>
#include "../src_f/f77-uscore.h"

#define PROGRAM "snowalbedo"
#define VERSION "1.0"

static void print_usage (char *name)
{
  fprintf (stderr, "\nusage:   %s MIE_ICE.tab [options]\n\n", name);
  fprintf (stderr, "%s  calculates the diffuse and direct albedo of snow as\n", name);
  fprintf (stderr, "formulated by Wiscombe and Warren, Journal of the\n");
  fprintf (stderr, "Atmospheric Sciences, vol, 37, 2712-2733, 1980.\n\n");
  fprintf (stderr, "Required arguments:\n");
  fprintf (stderr, "  -l            Equivalent depth of liquid water in snowpack (g cm -2)\n");
  fprintf (stderr, "  -r            mean grain radius (um)\n");
  fprintf (stderr, "  -u            cosine of solar zenith angle\n");
  fprintf (stderr, "Optional arguments:\n");
  fprintf (stderr, "  -a            albedo of underlying surface, default 0.03\n");
  fprintf (stderr, "  -h            prints this message\n");
  fprintf (stderr, "  -?            prints this message\n");
  fprintf (stderr, "  -p            turn of printing of messages\n");
  fprintf (stderr, "\n");
  fprintf (stderr, "The MIE_ICE.tab.?? files must first be generated using\n");
  fprintf (stderr, "the Gen_snow_tab.pl perl script. The use of Gen_snow_tab.pl\n");
  fprintf (stderr, "is shown when executing perl Gen_snow_tab.pl\n");
  fprintf (stderr, "Note that Gen_snow_tab.pl generates three files\n");
  fprintf (stderr, "with extensions .gg, .qext and .ssa.\n");
  fprintf (stderr, "snowalbedo should just be fed the name of the common \n");
  fprintf (stderr, "body of the three files, see example below \n");
  fprintf (stderr, "\n");
  fprintf (stderr, "For example  %s -p ../examples/MIE_ICE.tab -l 0.05 -r 50 -u 0.5 -p \n", PROGRAM);
  fprintf (stderr, "will create: (only two first output lines shown)\n");
  fprintf (stderr, "290.0 2.00893  0.9999776000 0.88037   0.9728   0.9689\n");
  fprintf (stderr, "291.0 2.01212  0.9999782400 0.88064   0.9731   0.9693\n");
  fprintf (stderr, "\n");
  fprintf (stderr, "Here,\n");
  fprintf (stderr, "1. Column: wavelength (nm)\n");
  fprintf (stderr, "2. Column: Q_ext\n");
  fprintf (stderr, "3. Column: Single scattering albedo\n");
  fprintf (stderr, "4. Column: Asymmetry parameter\n");
  fprintf (stderr, "5. Column: Direct albedo\n");
  fprintf (stderr, "6. Column: Diffuse albedo\n");
}

void snowalbedo(double omega, double tau, double gg, double surface_albedo, double umu0,
		       double* albedo_diffuse, double* albedo_direct) 
{
/* Function: snowalbedo                                                   @30_30i@ */
/* Description:                                                                    */
/*  Calculate the diffuse and direct albedo as formulated by Wiscombe and Warren,  */
/*  Journal of the Atmospheric Sciences, vol, 37, 2712-2733, 1980. Equation        */
/*  numbers below refer to equations in their paper.                               */
/*                                                                                 */
/* Parameters:                                                                     */
/*  double omega:      snow single scattering albedo                               */
/*  double tau:        snow optical depth                                          */
/*  double gg:         snow assymmetry factor                                      */
/*  double surface_albedo: albedo of underlying surface                            */
/*  double umu0:       cosine of solar zenith angle                                */
/*  double albedo_diffuse: diffuse snow albedo, Eq. 6                              */
/*  double albedo_direct: direct snow albedo, Eq. 3                                */
/*                                                                                 */
/* Return value:                                                                   */
/*  None                                                                           */
/*                                                                                 */
/* Example:                                                                        */
/* Files:                                                                          */
/* Known bugs:                                                                     */
/* Author: Arve Kylling, arve.kylling@@nilu.no                                      */
/*                                                                        @i30_30@ */

  double F77_FCN (dei) (double* x);
  double omegas, taus, ggs;
  double as, bs, chi, PP, gamma, QQP, QQM, QQ;
  double arg, dei1, dei2, dei3;

  /* Delta-Eddington scaling */
  ggs       = gg/(1.+gg);                           /* Eq. 2a */
  omegas    = ((1.-gg*gg)*omega)/(1.-gg*gg*omega);  /* Eq. 2b */
  taus      = (1.-gg*gg*omega)*tau;                 /* Eq. 2c */

  /* Definitions following Eq. 3. */
  as        = 1-omegas*ggs;
  bs        = ggs/as;
  chi       = sqrt(3*as*(1.-omegas));
  PP        = 2.*chi/(3*as);
  gamma     = (1.-surface_albedo)/(1.+surface_albedo);
  QQM       = (gamma-PP)*exp(-chi*tau);
  QQP       = (gamma+PP)*exp(+chi*tau);
  QQ        = (1.+PP)*QQP - (1.-PP)*QQM;
    
  /* Direct snow albedo, Eq. 3 */
  *albedo_direct = (2*(PP*(1.-gamma+omegas*bs) + 
		      omegas*(1.+bs)*(gamma*chi*umu0-PP)/(1.-chi*chi*umu0*umu0))*exp(-taus/umu0) 
		    - omegas*bs*(QQP - QQM)
		    + omegas*(1.+bs)*(QQP/(1.+chi*umu0) - QQM/(1.-chi*umu0) )
		    )/QQ;

  /* Diffuse snow albedo, Eq. 6 */
  arg = -taus;                     dei1 = F77_FCN (dei) (&arg);
  arg = -(1.+chi)*taus;            dei2 = F77_FCN (dei) (&arg);
  arg = -(1.-chi)*taus;            dei3 = F77_FCN (dei) (&arg);
  *albedo_diffuse = (2*PP*( (1.-gamma+omegas*bs)*(1.-taus)-(gamma*omegas)*(1.+bs)/(1.-omegas))*exp(-taus)
    		     - 2*PP*( omegas*(1.+bs)*(2./(chi*chi)+(gamma*taus)/(1-omegas))+
			      			      (1.-gamma+omegas*bs)*taus*taus)*dei1
		     + 2.*omegas*(1.+bs)/(chi*chi)*(QQP*(dei2 + chi - log(1.+chi)) -
							QQM*(dei3 - chi - log(fabs(1.-chi))))
		     - omegas*bs*(QQP-QQM)
		     )/QQ;

}


int main (int argc, char **argv)
{
  char filename[FILENAME_MAX+200] = "";
  char *file_qext, *file_ssa, *file_gg;
  int print =1;
  int status=0;
  int iv=0;
  int c;
  char *dummy=NULL;
  double omega, tau, gg, lambda; 
  double surface_albedo, umu0; 
  double albedo_diffuse;
  double albedo_direct;
  double LWD, Qext, r_mean, rho_ice;

  TABLE *table_qext = NULL, *table_ssa = NULL, *table_gg = NULL;

  rho_ice = 0.917; /* The density of ice, in g cm-3. */
  LWD     = 10.;   /* Equivalent depth of liquid water in snowpack, g cm-2. */
  Qext    = 2.01;
  r_mean  = 0.050;  /* Mean snow grain radius. */
  surface_albedo = 0.03;
  umu0 = 0.5;

  while ((c=getopt (argc, argv, "a:c?hl:pr:u:")) != EOF)  {
    switch(c)  {
    case 'a':   
      surface_albedo = strtod (optarg, &dummy);
      break;
    case 'l':   
      LWD = strtod (optarg, &dummy);
      break;
    case 'r':   
      r_mean = strtod (optarg, &dummy);
      break;
    case 'u':   
      umu0 = strtod (optarg, &dummy);
      break;
    case 'p':  /* no print messages */  
      print = 0;
      break;   
    case 'h':
      print_usage (argv[0]);
      return (-1);
      break;
    case '?':
      print_usage (argv[0]);
      return (-1);
      break;
    default:
      print_usage (argv[0]);
      return (-1);
    }
  }
      
  /* check number of remaining command line arguments */
  if (argc - optind != 1)  {
    fprintf(stderr,"No table file specified\n"); 
    print_usage (argv[0]);
    return -1;
  }
  
  strcpy (filename, argv[optind]);
  file_qext = (char *) calloc (strlen(filename)+6, sizeof (char));
  strcpy (file_qext, filename);
  file_qext = strncat (file_qext, ".qext", 5);

  file_ssa  = (char *) calloc (strlen(filename)+5, sizeof (char));
  strcpy (file_ssa, filename);
  file_ssa  = strncat (file_ssa, ".ssa", 4);

  file_gg   = (char *) calloc (strlen(filename)+4, sizeof (char));
  strcpy (file_gg, filename);
  file_gg   = strncat (file_gg, ".gg", 3);

  status = read_table (file_qext, &table_qext);
  if (status!=0) {
    switch (status) {
    case ASCIIFILE_NOT_FOUND:
      fprintf(stderr,"Table file not found %s\n",file_qext);
      break;
    default:
      fprintf(stderr,"Problem reading table file %s\n",file_qext);
      break;
    }
    exit(-1);
  }
  status = read_table (file_ssa, &table_ssa);
  if (status!=0) {
    switch (status) {
    case ASCIIFILE_NOT_FOUND:
      fprintf(stderr,"Table file not found %s\n",file_ssa);
      break;
    default:
      fprintf(stderr,"Problem reading table file %s\n",file_ssa);
      break;
    }
    exit(-1);
  }
  status = read_table (file_gg, &table_gg);
  if (status!=0) {
    switch (status) {
    case ASCIIFILE_NOT_FOUND:
      fprintf(stderr,"Table file not found %s\n",file_gg);
      break;
    default:
      fprintf(stderr,"Problem reading table file %s\n",file_gg);
      break;
    }
    exit(-1);
  }

  /* Output albedo for all wavelengths */
  for (iv=290;iv<=700;iv++) {
    lambda = (double) iv;
    status =  table_calculate (table_qext, r_mean, lambda, &Qext);
    status =  table_calculate (table_ssa,  r_mean, lambda, &omega);
    status =  table_calculate (table_gg,   r_mean, lambda, &gg);

    /* Factor 1.e-06 converts from um to cm for r_mean */
    tau   = (3.*LWD*Qext)/(4.*r_mean*1.e-06*rho_ice);
    
    snowalbedo( omega, tau, gg, surface_albedo, umu0, &albedo_diffuse, &albedo_direct);
    
    if ( print ) {
      fprintf (stderr, "Direct albedo:  %8.4f\nDiffuse albedo: %8.4f\n",
	       albedo_direct, albedo_diffuse);
    }
    else {
      printf ("%5.1f %7.5f %13.10f %7.5f %8.4f %8.4f\n",
	      lambda, Qext, omega, gg, albedo_direct, albedo_diffuse);
    }
  }
  return status;  
}
