/************************************************************************/
/*                                                                      */
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*----------------------------------------------------------------------*/
/************************************************************************/

#ifndef __uvspec_h
#define __uvspec_h

#include <stdio.h>

#define VERSION 0.12

/* Indices for the reading of the atmosphere files */
#define ATMFILE_ZD       0
#define ATMFILE_PRESS    1
#define ATMFILE_TEMPER   2
#define ATMFILE_AIR      3
#define ATMFILE_O3       4

#define AIR      0
#define O3       1
#define OTHER    2

#define O3_MOLINA    1
#define O3_DAUMONT   2
#define O3_BASSPAUR  3    

#define WCZD     0    
#define WCCON    1
#define WCEFFR   2

#define DISORT   1
#define SDISORT  2
#define TWOSTR   3
#define SUCORD   4

#define MAXAWVN 47

/* Raditive Transfer Equation STRUCTURE */
/* See disort.doc for description of variables */

typedef struct {
  int     deltam;
  int     fisot;
  int     nlev;
  int     nlyr;
  int     nstr;
  int     maxphi;
  int     maxumu;
  int     nphi;
  int     numu;
  int     solver;
  int     nprndis;
  int     reverse;
  int*    prndis;
  float*  phi;
  float*  umu;
} rte_struct;

/* Aerosol input STRUCTURE */

typedef struct {
  char*        tau_filename;
  int haze;
  int vulcan;
  int seasn;
  float alpha;
  float beta;
  float visib;
  float scale_ssa;
  float set_gg;
  float set_ssa;
} aer_inp_struct;

/* Aerosol output STRUCTURE */

typedef struct {
  float   *aer_pro;
  float   **dtau;
  float   **gg;
  float   **ssa;
} aer_out_struct;

/* Water Cloud input STRUCTURE */

typedef struct {
  float   wctau;  
} wc_inp_struct;

/* Water Cloud output STRUCTURE */

typedef struct {
  float   **dtau;
  float   **gg;
  float   **ssa;
  float   *wccon;
  float   *wceffr;
  float   *wczd;
  int     wcnlev;
  int     newsiz;
} wc_out_struct;

/* Ice Cloud STRUCTURE */

typedef struct {
  float   **dtau;
  float   **gg;
  float   **ssa;
} ice_struct;

/* Wavelength input grid STRUCTURE */

typedef struct {
  float   wl_start;
  float   wl_end;
} wl_inp_struct;

/* Wavelength output grid STRUCTURE */

typedef struct {
  int      nlambda_h;
  int      nlambda_s;
  int      nlambda_t;
  float*  lambda_h;
  float*  lambda_s;
  float*  lambda_t;
  float*  fbeam;
} wl_out_struct;

/* Atmosphere input data STRUCTURE */

typedef struct {
  float *zout;
  int    nzout;
  float  albedo;
  float  altitude;
  float  sza;
  float  phi0;
} atm_inp_struct;

/* Atmosphere output data STRUCTURE */

typedef struct {
  float **dens;
  float *press;
  float *rh;
  float *temper;
  float *temper_out;
  float *zd;
  float *file_albedo;
  float *file_lambda_albedo;
  float *albedo_t;
  float *file_phi0;
  float *file_sza;
  float *file_lambda_sza;
  float *sza_t;
  float *phi0_t;
  int    file_nlambda_albedo;
  int    nlambda_albedo;
  int    file_nlambda_sza;
  int    nlambda_sza;
  int    nlev;
  int    nlyr;
  int    ndens;
} atm_out_struct;

/* Cross section input data STRUCTURE */

typedef struct {
  char*  filename;
  int    o3_type;
  int    column;
} crs_inp_struct;

/* Cross section output data STRUCTURE */

typedef struct {
  double* ray;
} crs_out_struct;

/* INPUT STRUCTURE */

typedef struct {
  char*        filepath;
  char*        albedo_filename;
  char*        atmosphere_filename;
  char*        extraterrestrial_filename;
  char*        sza_filename;
  char*        transmittance_wl_file;
  char*        watercloud_filename;
  char*        slit_function_filename;
  int          convolve;
  int          spline;
  int          header;
  int          flexstor;
  float        spline_lambda_0;
  float        spline_lambda_1;
  float        spline_lambda_step;
  char*        spline_filename;
  int          day_of_year;
  float        ozone_column;
  float        pressure;
  rte_struct   rte;
  atm_inp_struct atm;
  aer_inp_struct   aer;
  crs_inp_struct   crs;
  ice_struct   ice;
  wl_inp_struct    wl;
  wc_inp_struct   wc;
} input_struct;


/* OUTPUT STRUCTURE */

typedef struct {
  int         phi;
} print_struct;

typedef struct {
  float eccentricity_factor;
  float       ozone_column;
  float ozone_scale_factor;
  float       ***u0u;
  float       ****uu;
  float       **flup;
  float       **dfdt;
  float       **rfldir;
  float       **rfldn;
  float       **uavgdn;
  float       **uavgso;
  float       **uavgup;
  float       **uavg;
  float       **flup_t;
  float       **dfdt_t;
  float       **rfldir_t;
  float       **rfldn_t;
  float       *sza;
  float       **uavgdn_t;
  float       **uavgso_t;
  float       **uavgup_t;
  float       **uavg_t;
  float       ***u0u_t;
  float       ****uu_t;
  atm_out_struct   atm;
  aer_out_struct   aer;
  crs_out_struct   crs;
  ice_struct   ice;
  wc_out_struct   wc;
  wl_out_struct    wl;
  print_struct    print;
} output_struct;

int uvspec(input_struct input, output_struct *output);
int uvspec_check(input_struct input);

#endif



