/*
 Copyright (C) 1997 Arve Kylling

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 1, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 To obtain a copy of the GNU General Public License write to the
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 USA.
*/

#include <stdio.h>							    
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../src_c/ascii.h"
#include "../src_c/fortran_and_c.h"
#include "../src_c/numeric.h"
#include "../src_c/radiative.h"
#include "../src_c/sun.h"
#include "../src_c/getopt.h"
#include "../src_f/f77-uscore.h"
#include "uvspec.h"
#include "uvspec_anc.h"

int read_atmosphere(char* atmosphere_filename, atm_inp_struct *atm_inp, 
		    atm_out_struct *atm_out, int other_column) 
{
  int lev;
  int rows=0, min_columns=0, max_columns=0;
  int status = 0;
  float **atm_data=NULL;
  float *dens=NULL;
  
  /*  printf("%s\n",input.atmosphere_filename); */

  status = ASCII_file2float (atmosphere_filename, 
			      &rows, &max_columns, &min_columns, 
			      &atm_data);
  if (status!=0) {
    fprintf (stderr, " ... error %d opening file %s\n", status, atmosphere_filename);
    return status;
  }
  if (max_columns!=min_columns) {
    fprintf (stderr, " !! ATTENTION !! Inconsistent number of columns\n");
    fprintf (stderr, "     min = %d, max =%d\n", min_columns, max_columns);
    return -1;
  }  
  if (min_columns<2) {
    fprintf (stderr, " ... ending, too few columns\n");
    return -1;
  }
  if (min_columns<other_column) {
    fprintf (stderr, " ... ending, too few columns\n");
    return -1;
  }
  /*  printf("atm: %d\n",other_column);*/
  
  atm_out->nlev = rows;
  atm_out->nlyr = rows-1;
  /*  printf("atm rows %d\n",rows);*/
  
  atm_out->press     = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_PRESS );
  atm_out->temper    = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_TEMPER );
  atm_out->zd        = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_ZD );
  if ( (status = ASCII_calloc_float (&atm_out->dens, atm_out->ndens, atm_out->nlev)) != 0 )
    return status;
  dens                = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_AIR );
  for (lev=0;lev<atm_out->nlev;lev++) { 
    atm_out->dens[AIR][lev] = dens[lev];
  }
  free(dens);  
  dens                = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_O3 );
  for (lev=0;lev<atm_out->nlev;lev++) { 
    atm_out->dens[O3][lev] = dens[lev];
  }
  free(dens);  

  /* allocate memory for OTHER in any case */
  atm_out->dens[OTHER]=calloc(atm_out->nlev, sizeof(float));

  /*  dens                = ASCII_column_float(atm_data, atm_out->nlev, other_column );
  for (lev=0;lev<atm_out->nlev;lev++) { 
    atm_out->dens[OTHER][lev] = dens[lev];
  }
  free(dens);*/

  status = ASCII_free_float(atm_data, rows);
  /*  printf("done read atmosphere\n");*/
  return 0;
}


/**************************************************************/
/* Shift struct atm_out_struct to specified altitude.         */
/**************************************************************/

int shift_atmosphere(atm_inp_struct *atm_inp, 
		     atm_out_struct *atm_out)
{
  int i=0, spline_stat=0;

  float *z_shift = calloc(atm_out->nlev, sizeof(float));
  
  float *zd     = calloc(atm_out->nlev, sizeof(float));
  float *press  = calloc(atm_out->nlev, sizeof(float));
  float *temper = calloc(atm_out->nlev, sizeof(float));
  float *rh     = calloc(atm_out->nlev, sizeof(float));
  float *air    = calloc(atm_out->nlev, sizeof(float));
  float *o3     = calloc(atm_out->nlev, sizeof(float));
  float *other  = calloc(atm_out->nlev, sizeof(float));

  float *p_0     =NULL, *p_1     =NULL, *p_2     =NULL, *p_3     =NULL;
  float *T_0     =NULL, *T_1     =NULL, *T_2     =NULL, *T_3     =NULL;
  float *rh_0    =NULL, *rh_1    =NULL, *rh_2    =NULL, *rh_3    =NULL;
  float *air_0   =NULL, *air_1   =NULL, *air_2   =NULL, *air_3   =NULL;
  float *o3_0    =NULL, *o3_1    =NULL, *o3_2    =NULL, *o3_3    =NULL;
  float *other_0 =NULL, *other_1 =NULL, *other_2 =NULL, *other_3 =NULL;


  /* sort profile in ascending order, assuming that */
  /* the original profile was descending            */
  for (i=0; i<atm_out->nlev; i++) {
    zd[i]       = atm_out->zd          [atm_out->nlev-1-i];
    press[i]    = atm_out->press       [atm_out->nlev-1-i];
    temper[i]   = atm_out->temper      [atm_out->nlev-1-i];
    rh[i]       = atm_out->rh          [atm_out->nlev-1-i];
    air[i]      = atm_out->dens[AIR]   [atm_out->nlev-1-i];
    o3[i]       = atm_out->dens[O3]    [atm_out->nlev-1-i];
    other[i]    = atm_out->dens[OTHER] [atm_out->nlev-1-i];
  }
  

  /* calculate shifted levels (altitude now in km) */
  for (i=0; i<atm_out->nlev; i++)
    z_shift[i] = zd[i] + atm_inp->altitude;


  /* calculate spline coefficients for profiles */
  spline_stat  = (char) fspline_coeffc (zd, press, atm_out->nlev, 
					&p_0, &p_1, &p_2, &p_3);
  
  spline_stat |= (char) fspline_coeffc (zd, temper,      atm_out->nlev, 
					&T_0, &T_1, &T_2, &T_3);

  spline_stat |= (char) fspline_coeffc (zd, rh,          atm_out->nlev, 
					&rh_0, &rh_1, &rh_2, &rh_3);

  spline_stat |= (char) fspline_coeffc (zd, air,   atm_out->nlev, 
					&air_0, &air_1, &air_2, &air_3);
  
  spline_stat |= (char) fspline_coeffc (zd, o3,    atm_out->nlev, 
					&o3_0, &o3_1, &o3_2, &o3_3);
  
  spline_stat |= (char) fspline_coeffc (zd, other, atm_out->nlev, 
					&other_0, &other_1, &other_2, &other_3);
  
  if (spline_stat!=0) {
    fprintf (stderr, " ... shift_atmosphere(): error %d calculating spline coefficients for\n", 
	     spline_stat);
    fprintf (stderr, "     one of the profiles\n"); 

    free(z_shift);
    free(press);
    free(temper);
    free(rh);
    free(air);
    free(o3);
    free(other);

    free(p_0);     free(p_1);     free(p_2);     free(p_3);
    free(T_0);     free(T_1);     free(T_2);     free(T_3);
    free(rh_0);    free(rh_1);    free(rh_2);    free(rh_3);
    free(air_0);   free(air_1);   free(air_2);   free(air_3);
    free(o3_0);    free(o3_1);    free(o3_2);    free(o3_3);
    free(other_0); free(other_1); free(other_2); free(other_3);

    return spline_stat;
  }

  

  /* Calculate splined values */

  for (i=0; i<atm_out->nlev; i++) {

    spline_stat |= (char) fcalc_splined_value (z_shift[i], &(press[i]), 
					       zd, atm_out->nlev, 
					       p_0, p_1, p_2, p_3);
    
    spline_stat |= (char) fcalc_splined_value (z_shift[i], &(temper[i]), 
					       zd, atm_out->nlev, 
					       T_0, T_1, T_2, T_3);
      
    spline_stat |= (char) fcalc_splined_value (z_shift[i], &(rh[i]), 
					       zd, atm_out->nlev, 
					       rh_0, rh_1, rh_2, rh_3);
    
    spline_stat |= (char) fcalc_splined_value (z_shift[i], &(air[i]), 
					       zd, atm_out->nlev, 
					       air_0, air_1, air_2, air_3);
    
    spline_stat |= (char) fcalc_splined_value (z_shift[i], &(o3[i]), 
					       zd, atm_out->nlev, 
					       o3_0, o3_1, o3_2, o3_3);
    
    spline_stat |= (char) fcalc_splined_value (z_shift[i], &(other[i]), 
					       zd, atm_out->nlev, 
					       other_0, other_1, other_2, other_3);

    
    /* if top of the atmosphere reached, cut profiles */ 
    if (spline_stat !=0) {
      atm_out->nlev = i;
      atm_out->nlyr = i-1;
      exit;
    }
  }  

  /* sort profile in decending order */
  for (i=0; i<atm_out->nlev; i++) {
    atm_out->zd          [atm_out->nlev-1-i] = zd[i];
    atm_out->press       [atm_out->nlev-1-i] = press[i];
    atm_out->temper      [atm_out->nlev-1-i] = temper[i];   
    atm_out->rh          [atm_out->nlev-1-i] = rh[i];       
    atm_out->dens[AIR]   [atm_out->nlev-1-i] = air[i];      
    atm_out->dens[O3]    [atm_out->nlev-1-i] = o3[i];       
    atm_out->dens[OTHER] [atm_out->nlev-1-i] = other[i];    
  }

  

  /* free memory */
  free(z_shift);
  free(press);
  free(temper);
  free(rh);
  free(air);
  free(o3);
  free(other);

  free(p_0);     free(p_1);     free(p_2);     free(p_3);
  free(T_0);     free(T_1);     free(T_2);     free(T_3);
  free(rh_0);    free(rh_1);    free(rh_2);    free(rh_3);
  free(air_0);   free(air_1);   free(air_2);   free(air_3);
  free(o3_0);    free(o3_1);    free(o3_2);    free(o3_3);
  free(other_0); free(other_1); free(other_2); free(other_3);


  return 0;  /* if o.k. */
}

int read_extraterrestrial(char* extraterrestrial_filename, 
			  wl_inp_struct *wl_inp, wl_out_struct *wl_out,
			  float wl_start, float wl_end) 
{
  int iv, rows=0, min_columns=0, max_columns=0, nlambda;
  int status = 0;
  float **wl_data=NULL;
  float *tmp_lambda=NULL, *tmp_fbeam=NULL;
  
  /*  printf("%s\n",input.atmosphere_filename); */

  status = ASCII_file2float (extraterrestrial_filename, 
			      &rows, &max_columns, &min_columns, 
			      &wl_data);
  if (status!=0) {
    fprintf (stderr, " ... error %d opening file %s\n", status, extraterrestrial_filename);
    return status;
  }
  if (max_columns!=min_columns) {
    fprintf (stderr, " !! ATTENTION !! Inconsistent number of columns\n");
    fprintf (stderr, "     min = %d, max =%d\n", min_columns, max_columns);
  }  
  if (min_columns<2) {
    fprintf (stderr, " ... ending, too few columns\n");
    return -1;
  }
  
  wl_out->nlambda_h = rows;

  /*  printf("atm rows %d\n",rows);*/
  
  tmp_lambda = ASCII_column_float(wl_data, wl_out->nlambda_h, 0 );
  tmp_fbeam  = ASCII_column_float(wl_data, wl_out->nlambda_h, 1 );

  nlambda = 0;
  for (iv=0;iv<wl_out->nlambda_h;iv++) {
    if ( (tmp_lambda[iv]>=wl_start) && (tmp_lambda[iv]<=wl_end) ) {
      nlambda++;
    }
  }

  wl_out->lambda_h = (float *) calloc (nlambda, sizeof(float));
  wl_out->fbeam     = (float *) calloc (nlambda, sizeof(float));

  nlambda = 0;
  for (iv=0;iv<wl_out->nlambda_h;iv++) {
    if ( (tmp_lambda[iv]>=wl_start) && (tmp_lambda[iv]<=wl_end) ) {
      wl_out->lambda_h[nlambda] = tmp_lambda[iv];
      wl_out->fbeam[nlambda]    = tmp_fbeam[iv];
      nlambda++;
    }
  }

  wl_out->nlambda_h=nlambda;
  free(tmp_lambda);
  free(tmp_fbeam);

  return 0;
}

int arb_wvn( int n_old_x, float *old_x, float *old_y,  
	      int n_new_x, float *new_x, float *new_y, int linear)
{
  int i, status;
  double *a0=NULL, *a1=NULL, *a2=NULL, *a3=NULL;
  double *x=NULL, *y=NULL;
  double ynew;

  x = (double *) calloc (n_old_x, sizeof(double));
  y = (double *) calloc (n_old_x, sizeof(double));
  for (i=0;i<n_old_x;i++) {
    x[i] = (double) old_x[i];
    y[i] = (double) old_y[i];
  }

  if ( linear ) {
    status = linear_coeffc (x, y, n_old_x, &a0, &a1, &a2, &a3);
    if (status!=0)  {
      fprintf (stderr, "arb_wvn: sorry cannot do linear interpolation\n");
      fprintf (stderr, "linear_coeffc() returned status %d\n", status);
      return status;
    }
  }
  else {
    status = spline_coeffc (x, y, n_old_x, &a0, &a1, &a2, &a3);
    if (status!=0)  {
      fprintf (stderr, "arb_wvn: sorry cannot do spline interpolation\n");
      fprintf (stderr, "spline_coeffc() returned status %d\n", status);
      return status;
    }
  }

  for (i=0; i<n_new_x; i++)  {
    status = calc_splined_value ((double) new_x[i], &ynew, x, n_old_x, a0, a1, a2, a3);
    new_y[i] = (float) ynew;
  }
  free(a0);
  free(a1);
  free(a2);
  free(a3);
  free(x);
  free(y);
  return 0;
}

int read_albedo(char* albedo_filename,  atm_inp_struct *atm_inp, atm_out_struct *atm_out) 
{
  int rows=0, min_columns=0, max_columns=0;
  int status = 0;
  float **alb_data=NULL;
  
  status = ASCII_file2float (albedo_filename, 
			      &rows, &max_columns, &min_columns, 
			      &alb_data);
  switch(status)  {
  case ASCIIFILE_NOT_FOUND: 
    fprintf (stderr, "Albedo file %s not found\n",  albedo_filename);
    return status;
    break;
  case ASCII_NO_MEMORY: 
    fprintf (stderr, "Not enough memory for reading albedo file %s\n",  albedo_filename);
    return status;
    break;
  case LESS_THAN_TWO_COLUMNS: 
    fprintf (stderr, "Albedo file %s has less than two columns\n",  albedo_filename);
    return status;
    break;
  default:
    break;
  }
  if (max_columns!=min_columns) {
    fprintf (stderr, "Inconsistent number of columns in file %s\n", albedo_filename);
    fprintf (stderr, "Columns min = %d, max =%d\n", min_columns, max_columns);
    status = 1;
    return status;
  }  
  if (min_columns<2) {
    fprintf (stderr, "Albedo file %s has less than two columns\n",  albedo_filename);
    status = 2;
    return status;
  }
  
  atm_out->file_nlambda_albedo = rows;

  /*  printf("atm rows %d\n",rows);*/
  
  atm_out->file_albedo = (float *) calloc (atm_out->file_nlambda_albedo, sizeof(float));
  atm_out->file_lambda_albedo = (float *) calloc (atm_out->file_nlambda_albedo, sizeof(float));

  atm_out->file_lambda_albedo = ASCII_column_float(alb_data, atm_out->file_nlambda_albedo, 0 );
  atm_out->file_albedo        = ASCII_column_float(alb_data, atm_out->file_nlambda_albedo, 1 );

  ASCII_free_float(alb_data,rows);

  return 0;
}

int read_aerosol_tau(char* aerosol_filename, int nlyr, int nlambda_t, 
		     float* zd, float** dtau) 
{
  int rows=0, min_columns=0, max_columns=0;
  int aero_nlyr=0, i, j;
  int status = 0;
  float **aero_data=NULL;
  float *aero_zd=NULL, *aero_tau=NULL;

  status = ASCII_file2float (aerosol_filename, 
			      &rows, &max_columns, &min_columns, 
			      &aero_data);
  switch(status)  {
  case ASCIIFILE_NOT_FOUND: 
    fprintf (stderr, "Aerosol tau file %s not found\n",  aerosol_filename);
    return status;
    break;
  case ASCII_NO_MEMORY: 
    fprintf (stderr, "Not enough memory for reading aerosol tau file %s\n",
	     aerosol_filename);
    return status;
    break;
  case LESS_THAN_TWO_COLUMNS: 
    fprintf (stderr, "Aerosol tau file %s has less than two columns\n",  
	     aerosol_filename);
    return status;
    break;
  default:
    break;
  }
  if (max_columns!=min_columns) {
    fprintf (stderr, "Inconsistent number of columns in file %s\n", aerosol_filename);
    fprintf (stderr, "Columns min = %d, max =%d\n", min_columns, max_columns);
    status = 1;
    return status;
  }  
  if (min_columns<2) {
    fprintf (stderr, "Aerosol tau file %s has less than two columns\n",
	     aerosol_filename);
    status = 2;
    return status;
  }
  
  aero_nlyr = rows-1;

  if (aero_nlyr != nlyr) {
    /*    printf("aero rows %d %d\n",rows, nlyr);*/
    fprintf (stderr, "Aerosol tau file %s: layering differs from atmospheric file\n", 
	     aerosol_filename); 
    status = 3;
    return status;
  }
  
  aero_zd  = (float *) calloc (nlyr, sizeof(float));
  aero_tau = (float *) calloc (nlyr, sizeof(float));

  aero_zd   = ASCII_column_float(aero_data, nlyr, 0 );
  aero_tau  = ASCII_column_float(aero_data, nlyr, 1 );

  ASCII_free_float(aero_data,rows);

  for (i=0;i<nlambda_t;i++) {
    for (j=0;j<nlyr;j++) {
      dtau[i][j] = aero_tau[j];
    }
  }

  return 0;
}

int read_sza(char* sza_filename,  atm_inp_struct *atm_inp, atm_out_struct *atm_out) 
{
  int i, rows=0, min_columns=0, max_columns=0;
  int status = 0;
  float **sza_data=NULL;
  
  status = ASCII_file2float (sza_filename, 
			      &rows, &max_columns, &min_columns, 
			      &sza_data);
  if (status!=0) {
    fprintf (stderr, " ... error %d opening file %s\n", status, sza_filename);
    return status;
  }
  if (max_columns!=min_columns) {
    fprintf (stderr, " !! ATTENTION !! Inconsistent number of columns\n");
    fprintf (stderr, "     min = %d, max =%d\n", min_columns, max_columns);
  }  
  if (min_columns<2) {
    fprintf (stderr, " ... ending, too few columns\n");
    return -1;
  }
  
  atm_out->file_nlambda_sza = rows;

  /*  printf("atm rows %d\n",rows);*/
  
  atm_out->file_sza = (float *) calloc (atm_out->file_nlambda_sza, sizeof(float));
  atm_out->file_phi0 = (float *) calloc (atm_out->file_nlambda_sza, sizeof(float));
  atm_out->file_lambda_sza = (float *) calloc (atm_out->file_nlambda_sza, sizeof(float));
  
  atm_out->file_lambda_sza = ASCII_column_float(sza_data, atm_out->file_nlambda_sza, 0 );
  atm_out->file_sza        = ASCII_column_float(sza_data, atm_out->file_nlambda_sza, 1 );
  if ( min_columns == 3) {
    atm_out->file_phi0       = ASCII_column_float(sza_data, atm_out->file_nlambda_sza, 2 );
  }
  else {
    for (i=0;i<atm_out->file_nlambda_sza;i++) {
      atm_out->file_phi0[i]  = 0.0;
    }
  }

  ASCII_free_float(sza_data,rows);

  return 0;
}

int set_transmittance_wl_grid(wl_inp_struct *wl_inp, wl_out_struct *wl_out)
{
  int iv;
  float lambda, wvn_step_t;
  
  /* Calculate number of wavelengths needed for transmittance calculation */

  lambda = wl_inp->wl_start;
  iv = 1;
  while (lambda < wl_inp->wl_end) {
    if ( lambda < 350.0 ) {
      wvn_step_t  = 0.5;
    }
    else {
      wvn_step_t  = 1.0;
	}
    lambda = lambda + wvn_step_t;
    iv++;
  }
  wl_out->nlambda_t = iv;
  /*    printf("Wl: %d\n",wl_out->nlambda_t);*/
  
  wl_out->lambda_t = (float *) calloc (wl_out->nlambda_t, sizeof(float));

  /* Set wavelengths for transmittance calculation */

  lambda = wl_inp->wl_start;
  for (iv=0;iv<wl_out->nlambda_t;iv++) {
    wl_out->lambda_t[iv] = lambda;
    if ( lambda < 350.0 ) {
      wvn_step_t  = 0.5;
    }
    else {
      wvn_step_t  = 1.0;
	}
    lambda = lambda + wvn_step_t;
  }
  return 0;
}

int convolve(input_struct input, output_struct *output)
{

  int iu, iv, j, lev;
  int status = 0;
  double *tmp=NULL, *x_data, *y_data;

  int n_slit;
  double  *x_slit, *y_slit;

  /*  printf("Convolution slit fucntion: %s\n", input.slit_function_filename);*/
 
  status = read_2c_file (input.slit_function_filename, &x_slit, &y_slit, &n_slit); 
  
  if (status!=0)  { 
    fprintf (stderr, " ... error %d reading file %s\n", status, input.slit_function_filename); 
    return status; 
  } 

 
  tmp = calloc (output->wl.nlambda_h, sizeof(double)); 
  x_data = calloc (output->wl.nlambda_h, sizeof(double)); 
  y_data = calloc (output->wl.nlambda_h, sizeof(double)); 
  for (iv=0;iv<output->wl.nlambda_h;iv++) {
    x_data[iv] = (double) output->wl.lambda_h[iv];
  }
  for (lev=0;lev<input.atm.nzout;lev++) {

    /* Convolve rfldir, rfldn, flup and uavg for all cases */

    /* rfldir */
    for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->rfldir[iv][lev]; }
    status = int_convolute ( x_data, y_data, output->wl.nlambda_h, x_slit, y_slit, n_slit, &tmp );
    for (iv=0;iv<output->wl.nlambda_h;iv++) { output->rfldir[iv][lev] = (float) tmp[iv]; }
          
    /* rfldn */
    for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->rfldn[iv][lev]; }
    status = int_convolute ( x_data, y_data, output->wl.nlambda_h, x_slit, y_slit, n_slit, &tmp );
    for (iv=0;iv<output->wl.nlambda_h;iv++) { output->rfldn[iv][lev] = (float) tmp[iv]; }

    /* flup */
    for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->flup[iv][lev]; }
    status = int_convolute ( x_data, y_data, output->wl.nlambda_h, x_slit, y_slit, n_slit, &tmp );
    for (iv=0;iv<output->wl.nlambda_h;iv++) { output->flup[iv][lev] = (float) tmp[iv]; }
          
    /* uavg */
    for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->uavg[iv][lev]; }
    status = int_convolute ( x_data, y_data, output->wl.nlambda_h, x_slit, y_slit, n_slit, &tmp );
    for (iv=0;iv<output->wl.nlambda_h;iv++) { output->uavg[iv][lev] = (float) tmp[iv]; }

    if (input.rte.solver != TWOSTR) {
      
      /* Convolve uavgso, uavgdn, uavgup */
      
      /* uavgso */
      for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->uavgso[iv][lev]; }
      status = int_convolute ( x_data, y_data, output->wl.nlambda_h, x_slit, y_slit, n_slit, &tmp );
      for (iv=0;iv<output->wl.nlambda_h;iv++) { output->uavgso[iv][lev] = (float) tmp[iv]; }
      
      /* uavgdn */
      for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->uavgdn[iv][lev]; }
      status = int_convolute ( x_data, y_data, output->wl.nlambda_h, x_slit, y_slit, n_slit, &tmp );
      for (iv=0;iv<output->wl.nlambda_h;iv++) { output->uavgdn[iv][lev] = (float) tmp[iv]; }
      
      /* uavgup */
      for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->uavgup[iv][lev]; }
      status = int_convolute ( x_data, y_data, output->wl.nlambda_h, x_slit, y_slit, n_slit, &tmp );
      for (iv=0;iv<output->wl.nlambda_h;iv++) { output->uavgup[iv][lev] = (float) tmp[iv]; }
      
      if ( input.rte.numu > 0 ) {
      
	/* Convolve u0u */
	for (iu=0;iu<input.rte.maxumu;iu++) {
	  for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->u0u[iv][lev][iu]; }
	  status = int_convolute ( x_data, y_data, output->wl.nlambda_h, x_slit, y_slit, n_slit, &tmp );
	  for (iv=0;iv<output->wl.nlambda_h;iv++) { output->u0u[iv][lev][iu] = (float) tmp[iv]; }
	}
	
	if ( output->print.phi > 0 ) {
	
	  /* Convolve uu */
	  for (j=0;j<input.rte.nphi;j++) {
	    for (iu=0;iu<input.rte.maxumu;iu++) {
	      for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->uu[iv][lev][j][iu]; }
	      status = int_convolute ( x_data, y_data, output->wl.nlambda_h, x_slit, y_slit, n_slit, &tmp );
	      for (iv=0;iv<output->wl.nlambda_h;iv++) { output->uu[iv][lev][j][iu] = (float) tmp[iv]; }
	    }
	  }
	}
      }
    }
  }

  free(tmp);  
  free(x_data);  
  free(y_data);  

  return status;
  
}

int spline_interpolate(input_struct input, output_struct *output)
{

  int iu, iv, j, lev;
  int status = 0;
  double xnew, ynew;
  double *x_data, *x_user=NULL, *y_data;
  double *a0=NULL, *a1=NULL, *a2=NULL, *a3=NULL;

  if ( strlen(input.spline_filename) > 0 ) {
    /* read file with user x values */
    status = read_1c_file (input.spline_filename, &x_user, &output->wl.nlambda_s);
    if (status!=0)  {
      fprintf (stderr, "spline_interpolate: error reading file %s\n", input.spline_filename);
      return (-1);
    }
    output->wl.lambda_s = (float *) calloc (output->wl.nlambda_s, sizeof(float));
    for (iv=0;iv<output->wl.nlambda_s;iv++) {
      output->wl.lambda_s[iv] = (float) x_user[iv];
      /*      printf("nlambda: %d %d %f\n", output->wl.nlambda_s, iv, output->wl.lambda_s[iv]);*/
    }
    /* Check wavelength ranges */
    if ( input.wl.wl_start > output->wl.lambda_s[0] )  {
      fprintf (stderr, "Spline lambda %f is smaller than wvn lambda %f\n",
	       output->wl.lambda_s[0], input.wl.wl_start);
      status++;
    }
    if ( input.wl.wl_end < output->wl.lambda_s[output->wl.nlambda_s-1] )  {
      fprintf (stderr, "Spline lambda %f is greater than wvn lambda %f\n",
	       output->wl.lambda_s[output->wl.nlambda_s-1], input.wl.wl_end);
      status++;
    }
    if (status>0) {return status;}
  }
  else {
    output->wl.nlambda_s = (int) (input.spline_lambda_1-input.spline_lambda_0)/input.spline_lambda_step+1;
    free(output->wl.lambda_s);
    output->wl.lambda_s = (float *) calloc (output->wl.nlambda_s, sizeof(float));
    for (iv=0;iv<output->wl.nlambda_s;iv++) {
      output->wl.lambda_s[iv] = input.spline_lambda_0 + iv*input.spline_lambda_step;
      /*      printf("nlambda: %d %d %f\n", output->wl.nlambda_s, iv, output->wl.lambda_s[iv]);*/
    }
  }
  
  /*  for (iv=0;iv<output->wl.nlambda_s;iv++) {
    printf("nlambda: %d %d %f\n", output->wl.nlambda_s, iv, output->wl.lambda_s[iv]);
  }*/

  /* Spline interpolate to given wavelengths */

  x_data = calloc (output->wl.nlambda_h, sizeof(double)); 
  y_data = calloc (output->wl.nlambda_h, sizeof(double)); 
  for (iv=0;iv<output->wl.nlambda_h;iv++) {
    x_data[iv] = (double) output->wl.lambda_h[iv];
  }
  for (lev=0;lev<input.atm.nzout;lev++) {

    /* Spline interpolate rfldir, rfldn, flup and uavg for all cases */
	  
    /* rfldir */
    for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->rfldir[iv][lev]; }
    status = spline_coeffc (x_data, y_data, output->wl.nlambda_h, &a0, &a1, &a2, &a3);
    if (status!=0)  {
      fprintf (stderr, "sorry cannot do spline interpolation\n");
      fprintf (stderr, "spline_coeffc() returned status %d\n", status);
    }
    for (iv=0; iv<output->wl.nlambda_s; iv++)  {
      xnew = (double) output->wl.lambda_s[iv];
      status = calc_splined_value (xnew, &ynew, x_data, output->wl.nlambda_h, a0, a1, a2, a3);
      if (status==0) { output->rfldir[iv][lev] = (float) ynew; } 
      else { output->rfldir[iv][lev] = NAN ; } 
    }
    free(a0); free(a1); free(a2); free(a3);

    /* rfldn */
    for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->rfldn[iv][lev]; }
    status = spline_coeffc (x_data, y_data, output->wl.nlambda_h, &a0, &a1, &a2, &a3);
    if (status!=0)  {
      fprintf (stderr, "sorry cannot do spline interpolation\n");
      fprintf (stderr, "spline_coeffc() returned status %d\n", status);
    }
    for (iv=0; iv<output->wl.nlambda_s; iv++)  {
      xnew = (double) output->wl.lambda_s[iv];
      status = calc_splined_value (xnew, &ynew, x_data, output->wl.nlambda_h, a0, a1, a2, a3);
      if (status==0) { output->rfldn[iv][lev] = (float) ynew; } 
      else { output->rfldn[iv][lev] = NAN ; } 
    }
    free(a0); free(a1); free(a2); free(a3);

    /* flup */
    for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->flup[iv][lev]; }
    status = spline_coeffc (x_data, y_data, output->wl.nlambda_h, &a0, &a1, &a2, &a3);
    if (status!=0)  {
      fprintf (stderr, "sorry cannot do spline interpolation\n");
      fprintf (stderr, "spline_coeffc() returned status %d\n", status);
    }
    for (iv=0; iv<output->wl.nlambda_s; iv++)  {
      xnew = (double) output->wl.lambda_s[iv];
      status = calc_splined_value (xnew, &ynew, x_data, output->wl.nlambda_h, a0, a1, a2, a3);
      if (status==0) { output->flup[iv][lev] = (float) ynew; } 
      else { output->flup[iv][lev] = NAN ; } 
    }
    free(a0); free(a1); free(a2); free(a3);

    /* uavg */
    for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->uavg[iv][lev]; }
    status = spline_coeffc (x_data, y_data, output->wl.nlambda_h, &a0, &a1, &a2, &a3);
    if (status!=0)  {
      fprintf (stderr, "sorry cannot do spline interpolation\n");
      fprintf (stderr, "spline_coeffc() returned status %d\n", status);
    }
    for (iv=0; iv<output->wl.nlambda_s; iv++)  {
      xnew = (double) output->wl.lambda_s[iv];
      status = calc_splined_value (xnew, &ynew, x_data, output->wl.nlambda_h, a0, a1, a2, a3);
      if (status==0) { output->uavg[iv][lev] = (float) ynew; } 
      else { output->uavg[iv][lev] = NAN ; } 
    }
    free(a0); free(a1); free(a2); free(a3);

    if (input.rte.solver != TWOSTR) {

      /* Spline interpolate uavgso, uavgdn, uavgup */
      
      /* uavgso */
      for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->uavgso[iv][lev]; }
      status = spline_coeffc (x_data, y_data, output->wl.nlambda_h, &a0, &a1, &a2, &a3);
      if (status!=0)  {
	fprintf (stderr, "sorry cannot do spline interpolation\n");
	fprintf (stderr, "spline_coeffc() returned status %d\n", status);
      }
      for (iv=0; iv<output->wl.nlambda_s; iv++)  {
	xnew = (double) output->wl.lambda_s[iv];
	status = calc_splined_value (xnew, &ynew, x_data, output->wl.nlambda_h, a0, a1, a2, a3);
	if (status==0) { output->uavgso[iv][lev] = (float) ynew; } 
	else { output->uavgso[iv][lev] = NAN ; } 
      }
      free(a0); free(a1); free(a2); free(a3);

      /* uavgdn */
      for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->uavgdn[iv][lev]; }
      status = spline_coeffc (x_data, y_data, output->wl.nlambda_h, &a0, &a1, &a2, &a3);
      if (status!=0)  {
	fprintf (stderr, "sorry cannot do spline interpolation\n");
	fprintf (stderr, "spline_coeffc() returned status %d\n", status);
      }
      for (iv=0; iv<output->wl.nlambda_s; iv++)  {
	xnew = (double) output->wl.lambda_s[iv];
	status = calc_splined_value (xnew, &ynew, x_data, output->wl.nlambda_h, a0, a1, a2, a3);
	if (status==0) { output->uavgdn[iv][lev] = (float) ynew; } 
	else { output->uavgdn[iv][lev] = NAN ; } 
      }
      free(a0); free(a1); free(a2); free(a3);

      /* uavgup */
      for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->uavgup[iv][lev]; }
      status = spline_coeffc (x_data, y_data, output->wl.nlambda_h, &a0, &a1, &a2, &a3);
      if (status!=0)  {
	fprintf (stderr, "sorry cannot do spline interpolation\n");
	fprintf (stderr, "spline_coeffc() returned status %d\n", status);
      }
      for (iv=0; iv<output->wl.nlambda_s; iv++)  {
	xnew = (double) output->wl.lambda_s[iv];
	status = calc_splined_value (xnew, &ynew, x_data, output->wl.nlambda_h, a0, a1, a2, a3);
	if (status==0) { output->uavgup[iv][lev] = (float) ynew; } 
	else { output->uavgup[iv][lev] = NAN ; } 
      }
      free(a0); free(a1); free(a2); free(a3);


      if ( input.rte.numu > 0 ) {
      
	/* Spline interpolate u0u */
	for (iu=0;iu<input.rte.maxumu;iu++) {
	  for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->u0u[iv][lev][iu]; }
	  status = spline_coeffc (x_data, y_data, output->wl.nlambda_h, &a0, &a1, &a2, &a3);
	  if (status!=0)  {
	    fprintf (stderr, "sorry cannot do spline interpolation\n");
	    fprintf (stderr, "spline_coeffc() returned status %d\n", status);
	  }
	  for (iv=0; iv<output->wl.nlambda_s; iv++)  {
	    xnew = (double) output->wl.lambda_s[iv];
	    status = calc_splined_value (xnew, &ynew, x_data, output->wl.nlambda_h, a0, a1, a2, a3);
	    if (status==0) { output->u0u[iv][lev][iu] = (float) ynew; } 
	    else { output->u0u[iv][lev][iu] = NAN ; } 
	  }
	  free(a0); free(a1); free(a2); free(a3);
	}

	if ( output->print.phi > 0 ) {
	
	  /* Spline interpolate uu */
	  for (j=0;j<input.rte.nphi;j++) {
	    for (iu=0;iu<input.rte.maxumu;iu++) {
	      for (iv=0;iv<output->wl.nlambda_h;iv++) {y_data[iv] = (double) output->uu[iv][lev][j][iu]; }
	      status = spline_coeffc (x_data, y_data, output->wl.nlambda_h, &a0, &a1, &a2, &a3);
	      if (status!=0)  {
		fprintf (stderr, "sorry cannot do spline interpolation\n");
		fprintf (stderr, "spline_coeffc() returned status %d\n", status);
	      }
	      for (iv=0; iv<output->wl.nlambda_s; iv++)  {
		xnew = (double) output->wl.lambda_s[iv];
		status = calc_splined_value (xnew, &ynew, x_data, output->wl.nlambda_h, a0, a1, a2, a3);
		if (status==0) { output->uu[iv][lev][j][iu] = (float) ynew; } 
		else { output->uu[iv][lev][j][iu] = NAN ; } 
	      }
	      free(a0); free(a1); free(a2); free(a3);
	    }
	  }
	}
      }
    }
  }
  

  free(output->wl.lambda_h);
  output->wl.nlambda_h = output->wl.nlambda_s;
  output->wl.lambda_h = (float *) calloc (output->wl.nlambda_s, sizeof(float));
  for (iv=0;iv<output->wl.nlambda_h;iv++) {
      output->wl.lambda_h[iv] = (float) output->wl.lambda_s[iv];
  }
  free(x_data);  
  free(y_data);  
  return status;
  
}


